#include <Xm/MainW.h>
#include <Xm/SelectioB.h>
#include <Xm/Text.h>
#include <Xm/FileSB.h>
#include <Xm/MessageB.h>
#include <X11/cursorfont.h>
#include <iostream>
#include <fstream>
#include <stdlib.h>
#include "global.h"

void resedit(char *text, char *title) {
  Widget toplevel, shell, main_w, menubar, f_menu, e_menu, edit_w;
  Arg    args[5];
  int    i;

  void file_cb(Widget widget, XtPointer client_data, XtPointer call_data);
  void edit_cb(Widget widget, XtPointer client_data, XtPointer call_data);

  toplevel=XtParent(StatData);
  while (!XtIsTopLevelShell(toplevel)) toplevel=XtParent(toplevel);

  shell  =XtVaCreatePopupShell("resedit",
			       topLevelShellWidgetClass, toplevel,
			       XtNtitle, title,
			       NULL);
  main_w =XtVaCreateWidget("main_w", xmMainWindowWidgetClass, shell, NULL);
  menubar=XmVaCreateSimpleMenuBar(main_w, "menubar",
				  XmVaCASCADEBUTTON, XmStringCreateLocalized("File"), 'F',
				  XmVaCASCADEBUTTON, XmStringCreateLocalized("Edit"), 'E',
				  NULL);
  f_menu=XmVaCreateSimplePulldownMenu(menubar, "file_menu", 0, file_cb,
			       XmVaPUSHBUTTON, XmStringCreateLocalized("Save"), 'S', NULL, NULL,
			       XmVaPUSHBUTTON, XmStringCreateLocalized("Append"), 'A', NULL, NULL,
			       XmVaSEPARATOR,
			       XmVaPUSHBUTTON, XmStringCreateLocalized("Print"), 'P', NULL, NULL,
			       XmVaSEPARATOR,
			       XmVaPUSHBUTTON, XmStringCreateLocalized("Close"), 'C', NULL, NULL,
			       NULL);
  e_menu=XmVaCreateSimplePulldownMenu(menubar, "edit_menu", 1, edit_cb,
	     XmVaPUSHBUTTON, XmStringCreateLocalized("Insert Pagebreak"), 'P', NULL, NULL,
	     NULL);
  i=0;
  XtSetArg(args[i], XmNrows, 50); i++;
  XtSetArg(args[i], XmNcolumns, 80); i++;
  XtSetArg(args[i], XmNeditMode, XmMULTI_LINE_EDIT); i++;
  edit_w=XmCreateScrolledText(main_w, "edit_w", args, i);
  XmTextSetString(edit_w, text);

  XtVaSetValues(f_menu, XmNuserData, edit_w, NULL);
  XtVaSetValues(e_menu, XmNuserData, edit_w, NULL);

  XtManageChild(edit_w);
  XtManageChild(menubar);
  XtManageChild(main_w);

  XtPopup(shell, XtGrabNone);
}

/*   .................   */
/*   c a l l b a c k s   */
/*   .................   */

void file_cb(Widget widget, XtPointer client_data, XtPointer call_data) {
  int    selection;
  Widget toplevel, edit_w, fsb_sd, dialog;
  Arg    args[5];
  char   cmd[255];
  int    i;

  void print_cb(Widget widget, XtPointer client_data, XtPointer call_data);
  void save_results_cb(Widget widget, XtPointer client_data, XtPointer call_data);
  extern void cancel_cb(Widget, XtPointer, XtPointer);
  
  XtVaGetValues(XtParent(widget), XmNuserData, &edit_w, NULL);
  selection=(int) client_data;

  toplevel=XtParent(widget);
  while (!XtIsTopLevelShell(toplevel)) toplevel=XtParent(toplevel);

  switch (selection) {
  case 0: /* save a file */;
    fsb_sd=XmCreateFileSelectionDialog(toplevel, "filesb", NULL, 0);
    XtAddCallback(fsb_sd,XmNokCallback, save_results_cb, edit_w);
    XtAddCallback(fsb_sd,XmNcancelCallback, cancel_cb, NULL);
    XtVaSetValues(XtParent(fsb_sd),
		  XmNtitle, "Save Results on File",
		  NULL);
    XtVaSetValues(fsb_sd, XmNuserData, 0, NULL); // 0 = overwrite;
    XtManageChild(fsb_sd);
    break;
  case 1:  /* append to file */
    fsb_sd=XmCreateFileSelectionDialog(toplevel, "filesb", NULL, 0);
    XtAddCallback(fsb_sd,XmNokCallback, save_results_cb, edit_w);
    XtAddCallback(fsb_sd,XmNcancelCallback, cancel_cb, NULL);
    XtVaSetValues(XtParent(fsb_sd),
		  XmNtitle, "Append Results to File",
		  NULL);
    XtVaSetValues(fsb_sd, XmNuserData, 1, NULL); // 1 = append;
    XtManageChild(fsb_sd);
    break;
  case 2: /* print a file */;
    i=0;
    XtSetArg(args[i], XmNselectionLabelString, XmStringCreateLocalized("Enter Print Command: ")); i++;
    XtSetArg(args[i], XmNtitle, "Printing"); i++;
    dialog=XmCreatePromptDialog(edit_w, "dialog", args, i);
    strcpy(cmd, PRINTCMD);
    XtVaSetValues(dialog,
		  XmNtextString, XmStringCreateLocalized(cmd),
		  NULL);
    XtAddCallback(dialog, XmNokCallback, print_cb, edit_w);
    XtUnmanageChild(XmSelectionBoxGetChild(dialog, XmDIALOG_HELP_BUTTON));

    XtManageChild(dialog);
    XtPopup(XtParent(dialog), XtGrabNone);
    break;
  case 3: /* close editor */;
    while(widget && !XtIsWMShell(widget)) widget=XtParent(widget);
    XtPopdown(widget);
    XtDestroyWidget(widget);
    break;
  }
}

void edit_cb(Widget widget, XtPointer client_data, XtPointer call_data) {
  Widget         edit_w;
  int            selection;
  char           *inset;
  XmTextPosition pos;

  selection=(int) client_data;
  XtVaGetValues(XtParent(widget), XmNuserData, &edit_w, NULL);

  switch (selection) {
  case 0: /* insert pagebreak */;
    inset = new char[5];
    strcpy(inset, "\f\n");
    pos=XmTextGetCursorPosition(edit_w);
    XmTextInsert(edit_w, pos, inset);
    delete[]inset;
    break;
  }
}

/* print callback */
void print_cb(Widget widget, XtPointer client_data, XtPointer call_data) {
  Widget edit_w;
  FILE   *pdat;
  XmSelectionBoxCallbackStruct *cbs=(XmSelectionBoxCallbackStruct *) call_data;
  char   *text, cmd[255];
  long   len;

  /* build temporary print file */
  edit_w=(Widget)client_data;
  text=XmTextGetString(edit_w);
  len=XmTextGetLastPosition(edit_w);
  pdat=fopen(PRINTDAT, "w");
  fwrite(text, sizeof(char), len, pdat);
  fputc('\n', pdat);
  fclose(pdat);
  XtFree(text);

  /* get print command and print */
  XmStringGetLtoR(cbs->value, XmFONTLIST_DEFAULT_TAG, &text);
  strcpy(cmd, text);
  strcat(cmd, " -s -r ");
  strcat(cmd, PRINTDAT);
  system(cmd);
  XtFree(text);
}


/* save results callback */
void save_results_cb(Widget widget, XtPointer client_data, XtPointer call_data) {
  Widget       edit_w, fexist;
  Cursor       uhr, pfeil;
  Arg          args[5];
  int          i, append;
  XmString     fname;
  char         *filename, *out_buf;
  std::fstream outdat;
  extern int   errno;

  void kill_resfile_cb(Widget widget, XtPointer client_data, XtPointer call_data);

  edit_w=(Widget) client_data;
  XtVaGetValues(widget, XmNuserData, &append, NULL); // 0=overwrite, 1=append;

  /* get filename */
  XtVaGetValues(widget, XmNdirSpec, &fname, NULL);
  if (!XmStringGetLtoR(fname, XmFONTLIST_DEFAULT_TAG, &filename)) {
    std::cout << "Internal error pasting filename! \n";
    return;
  }
  if (!*filename) {
    std::cout << "No file selected! \n";
    XtFree(filename);
    return;
  }

  /* test whether file already exists */
  if (!append) {
    outdat.open(filename, std::ios::in);
    if (outdat) {
      outdat.close();
      i=0;
      XtSetArg(args[i], XmNmessageString, XmStringCreateLtoR("File already exists.\n     Overwrite?",XmFONTLIST_DEFAULT_TAG)); i++;
      XtSetArg(args[i], XmNokLabelString, XmStringCreateLocalized(" Yes ")); i++;
      XtSetArg(args[i], XmNcancelLabelString, XmStringCreateLocalized(" No ")); i++;
      XtSetArg(args[i], XmNtitle, "Save Results"); i++;
      XtSetArg(args[i], XmNuserData, edit_w); i++;
      fexist=XmCreateQuestionDialog(XtParent(widget), "fexist", args, i);
      XtAddCallback(fexist, XmNokCallback, kill_resfile_cb, widget);
      XtUnmanageChild(XmMessageBoxGetChild(fexist, XmDIALOG_HELP_BUTTON));
      XtManageChild(fexist);
      XtPopup(XtParent(fexist), XtGrabNone);
    }
  }

  /* file does not exist - now write data */
  if (!outdat || append) {

    /* set cursor */
    uhr  =XCreateFontCursor(XtDisplay(widget),XC_watch);
    pfeil=XCreateFontCursor(XtDisplay(widget), XC_arrow);
    XDefineCursor(XtDisplay(widget),XtWindow(widget),uhr);
    XFlush(XtDisplay(widget));

    // validate opening of file;
    outdat.open(filename, std::ios::app);
    if (!outdat) {
      std::cout << "Can't open file: " << filename << "\n";
      }
    else {
      if (out_buf=XmTextGetString(edit_w)) {
	outdat << out_buf;
	XtFree(out_buf);
      }
    }

    outdat.close();
    XDefineCursor(XtDisplay(widget),XtWindow(widget),pfeil);
    XFreeCursor(XtDisplay(widget), pfeil);
    XFreeCursor(XtDisplay(widget), uhr);
    XtUnmanageChild(widget);
  }
  
  XtFree(filename);

}


void kill_resfile_cb(Widget widget, XtPointer client_data, XtPointer call_data) {
  Widget   fsb, edit_w;
  XmString fname;
  char     *filename, *cmd;

  cmd=new char[256];
  XtVaGetValues(widget, XmNuserData, &edit_w, NULL);
  fsb=(Widget) client_data;
  XtVaGetValues(fsb, XmNdirSpec, &fname, NULL);
  if (XmStringGetLtoR(fname, XmFONTLIST_DEFAULT_TAG, &filename)) {
    sprintf(cmd, "rm %s", filename);
    system(cmd);
    XtFree(filename);
    save_results_cb(fsb, (XtPointer)edit_w, NULL);
  }
  delete[]cmd;
}
